def human_bytes(n):
    """
    Return the number of bytes n in more human readable form.
    """
    if n < 1024:
        return '%d B' % n
    k = n/1024
    if k < 1024:
        return '%d KB' % round(k)
    m = k/1024
    if m < 1024:
        return '%.1f MB' % m
    g = m/1024
    return '%.2f GB' % g



"""
catimg

Uses iTerm2's proprietary escape codes and Imgur to display an image of a cat
in your terminal.

NOTE: I do not own the images that you see, nor have I any control over
them. You will see some image that is tagged as "cat" on Imgur. That could be
anything. I do filter out images that are tagged NSFW, but there are no
guarantees that you won't see something you wish you hadn't. Use at your own
risk.

License: MIT

"""
from .iterm2 import *
from .imgur import *

__version__ = '1.0.2'



import sys
import argparse

from .iterm2 import iterm2_display_image_file
from .imgur import update_img_cache, get_random_image

def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('-f', '--files', nargs='+', metavar='IMAGES', default=(),
                   help='Files to display')
    parser.add_argument('-v', '--verbose', action='store_true', help="Show verbose output")
    parser.add_argument('--update-cache', action='store_true', help="Update the image cache and exit")
    parser.add_argument('--no-download', action='store_false', dest='download', default=True, help="Don't download anything from the internet")
    parser.add_argument('--no-delete', action='store_false', dest='delete',
    default=True, help="Don't delete old images")

    args = parser.parse_args()

    for img in args.files:
        iterm2_display_image_file(img)
        print()
    if not args.files:
        if args.update_cache:
            update_img_cache(verbose=args.verbose)
            return
        image = get_random_image(verbose=args.verbose)
        if not image and args.download:
            print("No cat images found, downloading...")
            update_img_cache(verbose=args.verbose)
            image = get_random_image(delete=args.delete, verbose=args.verbose)
        if image:
            iterm2_display_image_file(image)
            print()

if __name__ == '__main__':
    sys.exit(main())



import sys
import os
import base64

# See https://iterm2.com/images.html
IMAGE_CODE= '\033]1337;File={file};inline={inline};size={size}:{base64_img}\a'

def iterm2_image_bytes(b, filename=None, inline=1):
    data = {
        'file': base64.b64encode((filename or 'Unnamed file').encode('utf-8')).decode('ascii'),
        'inline': inline,
        'size': len(b),
        'base64_img': base64.b64encode(b).decode('ascii'),
        }
    return (IMAGE_CODE.format(**data))

def iterm2_display_image_file(fn):
    """
    Display an image in the terminal.

    A newline is not printed.
    """
    with open(os.path.realpath(os.path.expanduser(fn)), 'br') as f:
        sys.stdout.write(iterm2_image_bytes(f.read(), filename=fn))



import sys
import os
import random

from collections import defaultdict
from concurrent.futures import ThreadPoolExecutor

from requests import Session

from imgurpython import ImgurClient
from imgurpython.helpers.error import ImgurClientError
from imgurpython.imgur.models.gallery_album import GalleryAlbum

from .util import human_bytes

CLIENT_ID = "6efbafee90d77dc"
CLIENT_SECRET = "d0e87b8e5ec48cb345380f6fbcaf2ff70167755c"

CATIMG_DIR = os.path.expanduser('~/.catimg')
USAGES_FILE = os.path.join(CATIMG_DIR, 'usages')
IMG_CACHE = os.path.join(CATIMG_DIR, 'cache')
LOCK_PATH = os.path.join(CATIMG_DIR, 'lock')
# client = ImgurClient(CLIENT_ID, CLIENT_SECRET)

def _download_and_write(item, path, session, verbose=False):
    print("Downloading %s (%s)" % (item.link, human_bytes(item.size)))
    r = session.get(item.link)

    if verbose:
        print("Writing %s" % path)
    with open(path, 'wb') as f:
        f.write(r.content)

def update_img_cache(verbose=False):
    try:
        os.makedirs(LOCK_PATH)
    except OSError:
        if verbose:
            print("Another process is downloading cat images.")
        return

    try:
        client = ImgurClient(CLIENT_ID, CLIENT_SECRET)
        if verbose:
            print("Getting Imgur images")
        items = client.gallery_tag('cat')
        session = Session()
        os.makedirs(IMG_CACHE, exist_ok=True)
        to_download = {}
        for item in items.items:
            fn = item.link.rsplit('/', 1)[1]
            path = os.path.join(IMG_CACHE, fn)

            if item.nsfw:
                if verbose:
                    print("%s is NSFW, skipping" % item.link)
                continue
            if isinstance(item, GalleryAlbum):
                if verbose:
                    print("%s is an album, skipping" % item.link)
                continue
            if os.path.exists(path):
                if verbose:
                    print("%s already exists, skipping" % path)
                continue
            to_download[item] = path

        with ThreadPoolExecutor(10) as executor:
            for item, path in to_download.items():
                executor.submit(_download_and_write, item, path, session,
            verbose=verbose)




    except ImgurClientError as e:
        print("Error: Could not get new images (%s)" % e, file=sys.stderr)
    finally:
        try:
            os.rmdir(LOCK_PATH)
        except FileNotFoundError:
            pass

def get_random_image(n=3, delete=True, verbose=False):
    """
    Get a random image, which hasn't been used more than n times. Usages are
    stored on disk.

    If delete=True, delete files that have been used more than n times.

    Returns the file path, or None if no files could be found.
    """
    if verbose:
        print("Max usage:", n)
    usages = defaultdict(int)
    os.makedirs(IMG_CACHE, exist_ok=True)
    if os.path.exists(USAGES_FILE):
        with open(USAGES_FILE, 'r') as f:
            for line in f.read().splitlines():
                usages.update({img: int(uses) for img, uses in [line.split()]})

    if verbose:
        print("Usages:", usages)

    files = os.listdir(IMG_CACHE)
    usable_files = {f for f in files if usages[f] < n}
    if verbose:
        print("Usable files:", usable_files)

    if not usable_files:
        return None

    random_file = random.choice(list(usable_files))
    if verbose:
        print("Random file:", random_file)
    usages[random_file] += 1

    if delete:
        # Shouldn't delete the random_file
        unusable_files = set(files) - usable_files
        for f in unusable_files:
            if verbose:
                print("Removing %s" % os.path.join(IMG_CACHE, f))
            os.remove(os.path.join(IMG_CACHE, f))

    if verbose:
        print("New usages:", usages)
        print("Writing usages to %s" % USAGES_FILE)
    with open(USAGES_FILE, 'w') as f:
        f.write('\n'.join(' '.join([img, str(usages[img])]) for img in usages))

    return os.path.join(IMG_CACHE, random_file)



#!/usr/bin/env python

import sys
if sys.version_info < (3,):
    sys.exit("catimg requires Python 3")

from setuptools import setup
import catimg

setup(
    name='catimg',
    version=catimg.__version__,
    description='''Print an image of a cat from Imgur to iTerm2.''',
    author='Aaron Meurer',
    author_email='asmeurer@gmail.com',
    url='https://github.com/asmeurer/catimg',
    packages=['catimg'],
    package_data={'catimg.tests': ['aloha_cat.png']},
    long_description=catimg.__doc__,
    entry_points={'console_scripts': [ 'catimg = catimg.__main__:main']},
    install_requires=[
        'requests',
        'imgurpython',
    ],
    license="MIT",
    classifiers=[
        'Environment :: MacOS X',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 3',
        ],
)
